@@ -4,6 +4,7 @@ import platform
 from .table import Table
 from .settings import config
 from .errors import DuplicateError
+from .heading import Heading
 
 ERROR_MESSAGE_LENGTH = 2047
 TRUNCATION_APPENDIX = '...truncated'
@@ -13,18 +14,17 @@ class JobTable(Table):
     """
     A base relation with no definition. Allows reserving jobs
     """
-    def __init__(self, arg, database=None):
-        if isinstance(arg, JobTable):
-            super().__init__(arg)
-            # copy constructor
-            self.database = arg.database
-            self._connection = arg._connection
-            self._definition = arg._definition
-            self._user = arg._user
-            return
-        super().__init__()
+    def __init__(self, conn, database):
         self.database = database
-        self._connection = arg
+        self._connection = conn
+        self._heading = Heading(table_info=dict(
+            conn=conn,
+            database=database,
+            table_name=self.table_name,
+            context=None
+        ))
+        self._support = [self.full_table_name]
+
         self._definition = """    # job reservation table for `{database}`
         table_name  :varchar(255)  # className of the table
         key_hash  :char(32)  # key hash
@@ -32,7 +32,7 @@ class JobTable(Table):
         status  :enum('reserved','error','ignore')  # if tuple is missing, the job is available
         key=null  :blob  # structure containing the key
         error_message=""  :varchar({error_message_length})  # error message returned if failed
-        error_stack=null  :blob  # error stack if failed
+        error_stack=null  :mediumblob  # error stack if failed
         user="" :varchar(255) # database user
         host=""  :varchar(255)  # system hostname
         pid=0  :int unsigned  # system process id
